#!/usr/bin/perl

use Benchmark qw(cmpthese);
use CGI::Ex::Dump qw(debug);
CGI::Ex::Dump::set_deparse();
my $n = -2;

my %DISPATCH;
$DISPATCH{'mysig'} = [sub { one(@_) },
                      sub { two(@_) },
                      sub { three(@_) }];
$DISPATCH{'mysig2'} = [sub { one(@_) }];
$DISPATCH{'empty'}  = [];

sub dispatch {
  my $sig = shift;
  return if ! exists $DISPATCH{$sig};
  &$_(@_) for @{ $DISPATCH{$sig} };
}

sub create_alias {
  my $sig = shift;
  return sub { &$_(@_) for @{ $DISPATCH{$sig} } }
}

my $aliased_mysig  = create_alias('mysig');
my $aliased_mysig2 = create_alias('mysig2');
my $aliased_empty  = create_alias('empty');

#my $x;
#sub one { $x .= "one (@_)\n" }
#sub two { $x .= "two (@_)\n" }
#sub three { $x .= "three (@_)\n" }
my $x = 0;
sub one { $x++ }
sub two { $x++ }
sub three { $x++ }

#dispatch('mysig', 'hello');
#&$aliased_mysig2('hello');
#print $x;
#exit;

cmpthese($n, {
  aliased_1 => sub {
    &$aliased_mysig('hello');
  },
  aliased_2 => sub {
    &$aliased_mysig2('hello');
  },
  aliased_e => sub {
    &$aliased_empty('hello');
  },
  dispatch_1 => sub {
    dispatch('mysig', 'hello');
  },
  dispatch_2 => sub {
    dispatch('mysig2', 'hello');
  },
  dispatch_e => sub {
    dispatch('empty', 'hello');
  },
  no_dis_1 => sub {
    one('hello');
    two('hello');
    three('hello');
  },
  no_dis_2 => sub {
    one('hello');
  },
  dispatch_miss => sub {
    dispatch('fooboo', 'hello');
  },
}, 'auto');

__END__

[pauls@localhost perl]$ perl ui/bench/dispatch.pl
Benchmark: running aliased_1, aliased_2, aliased_e, dispatch_1, dispatch_2, dispatch_e, dispatch_miss, no_dis_1, no_dis_2 forat least 2 CPU seconds...
  aliased_1:  2 wallclock secs ( 2.11 usr +  0.00 sys =  2.11 CPU) @ 108706.16/s (n=229370)
  aliased_2:  3 wallclock secs ( 2.11 usr +  0.00 sys =  2.11 CPU) @ 203824.64/s (n=430070)
  aliased_e:  1 wallclock secs ( 2.12 usr +  0.00 sys =  2.12 CPU) @ 502952.36/s (n=1066259)
dispatch_1:  2 wallclock secs ( 2.10 usr +  0.00 sys =  2.10 CPU) @ 96375.71/s (n=202389)
dispatch_2:  2 wallclock secs ( 2.03 usr +  0.01 sys =  2.04 CPU) @ 163637.25/s (n=333820)
dispatch_e:  1 wallclock secs ( 2.03 usr +  0.00 sys =  2.03 CPU) @ 328887.19/s (n=667641)
dispatch_miss:  1 wallclock secs ( 2.05 usr +  0.00 sys =  2.05 CPU) @ 545960.49/s (n=1119219)
  no_dis_1:  2 wallclock secs ( 2.13 usr +  0.00 sys =  2.13 CPU) @ 362798.59/s (n=772761)
  no_dis_2:  3 wallclock secs ( 2.09 usr +  0.00 sys =  2.09 CPU) @ 1097492.34/s (n=2293759)
                   Rate dispatch_1 aliased_1 dispatch_2 aliased_2 dispatch_e no_dis_1 aliased_e dispatch_miss no_dis_2
dispatch_1      96376/s         --      -11%       -41%      -53%       -71%     -73%      -81%          -82%     -91%
aliased_1      108706/s        13%        --       -34%      -47%       -67%     -70%      -78%          -80%     -90%
dispatch_2     163637/s        70%       51%         --      -20%       -50%     -55%      -67%          -70%     -85%
aliased_2      203825/s       111%       88%        25%        --       -38%     -44%      -59%          -63%     -81%
dispatch_e     328887/s       241%      203%       101%       61%         --      -9%      -35%          -40%     -70%
no_dis_1       362799/s       276%      234%       122%       78%        10%       --      -28%          -34%     -67%
aliased_e      502952/s       422%      363%       207%      147%        53%      39%        --           -8%     -54%
dispatch_miss  545960/s       466%      402%       234%      168%        66%      50%        9%            --     -50%
no_dis_2      1097492/s      1039%      910%       571%      438%       234%     203%      118%          101%       --
[pauls@localhost perl]$ fg
