#!/usr/bin/perl -w

{
  package Foo;
  use UI::Signals qw(attach detach);
  use UI::Signals signals => {add_it => 'number'};
  sub new {
    my $class = shift || __PACKAGE__;
    my $n     = shift || 0;
    bless \ $n, $class;
  }
  sub add { ${ +shift } += shift || 0 }
  sub n :lvalue { ${ +shift } }
}
{
  package Bar;
  @Bar::ISA = qw(Foo);
}
{
  package Baz;
  @Baz::ISA = qw(Bar);
}

###----------------------------------------------------------------###

use strict;

my $foo = Foo->new(1);
my $bar = Bar->new(2);
my $baz = Baz->new(3);
$foo->n = $bar->n = $baz->n = 0;

$foo->attach('one_it', $foo, 'add');
$foo->attach('two_it', $foo, 'add');
$foo->attach('two_it', $bar, 'add');
$foo->attach('three_it', $foo, 'add');
$foo->attach('three_it', $bar, 'add');
$foo->attach('three_it', $baz, 'add');


use Benchmark qw(cmpthese);
cmpthese(-2, {
  empty     => sub { $foo->add_it(1) },
  one_sig   => sub { $foo->one_it(1) },
  two_sig   => sub { $foo->two_it(1) },
  three_sig => sub { $foo->three_it(1) },
  one_no   => sub { $foo->add(1) },
  two_no   => sub { $foo->add(1); $bar->add(1) },
  three_no => sub { $foo->add(1); $bar->add(1); $baz->add(1) },
}, 'auto');
exit;

require Test::More;
import  Test::More tests => 4;

### attach three subs to signals
$foo->attach('add_it', $foo, 'add'); # method name
$foo->attach('add_it', $bar, sub {my $self = shift; $self->n = shift}); # code method
$foo->attach('add_it', undef, sub {$baz->n = shift}); # code ref

$foo->add_it(2);
ok($foo->n == 2 && $bar->n == 2 && $baz->n == 2, "Signal Test");
$foo->n = $bar->n = $baz->n = 0;

#UI::Signals->dump_signals;
$foo->detach('add_it', $foo);
$foo->add_it(2);
ok($foo->n == 0 && $bar->n == 2 && $baz->n == 2, "Signal Test 2");
$foo->n = $bar->n = $baz->n = 0;

$foo->detach('add_it', $bar);
$foo->add_it(2);
ok($foo->n == 0 && $bar->n == 0 && $baz->n == 2, "Signal Test 3");
$foo->n = $bar->n = $baz->n = 0;

$foo->detach('add_it');
$foo->add_it(2);
ok($foo->n == 0 && $bar->n == 0 && $baz->n == 0, "Signal Test 4");
$foo->n = $bar->n = $baz->n = 0;

#UI::Signals->dump_signals;
exit;

