package UI::All::CursesUI;

###----------------------------------------------------------------###
#  Copyright 2005 - Paul Seamons                                     #
#  Distributed under the Perl Artistic License without warranty      #
###----------------------------------------------------------------###

use Curses::UI;
use base qw(UI::All::Base);

sub is_term { 1 }

sub attach_proxy {}

###----------------------------------------------------------------###

package UI::All::CursesUI::MainWindow;
use base qw(UI::All::CursesUI);
use Curses::UI;

sub init_proxy {
  my $self = shift;
  my $argv = [];
  my %args;

  if (1) { #eval { require POSIX } && (my $file = POSIX::tmpnam())) {
    $file = "/tmp/curse.out";
    open (STDERR, ">>$file");
    select STDERR;
    $| = 1;
    select STDOUT;
    #END { open(_IN, $file); print <_IN>; unlink $file };
  }

  $self->add_property(app => Curses::UI->new(-clear_on_exit => 1,
                                             -color_support => 1,
                                             #-debug => 1,
                                             ));
  $self->proxy = $self->app->add(
                                 $self->name || $self->gen_name,
                                 'Window',
                                 # -centered => 1,
                                 );
}

sub parent { undef } # MainWindows shouldn't have parents

sub set_title { shift->proxy->title(shift) }

sub main_loop {
  my $self = shift;
  $self->app->mainloop;
}

sub exit_loop {
  my $self = shift;
  $self->DESTROY;
  $self->show_track;
  exit;
}

###----------------------------------------------------------------###

package Curses::UI::LayoutHBox;
use base qw(Curses::UI::Container);
$INC{'Curses/UI/LayoutHBox.pm'} = 'internal';

sub add {
  my $self = shift;
  my ($name, $type, %ARGS) = @_;
  my $widget = $self->SUPER::add($name, $type, %ARGS);
  $widget->{-priority} = $ARGS{-priority} if exists $ARGS{-priority};
  $self->layout_contained_objects;
  return $widget;
}

sub layout_contained_objects {
  my $self = shift;

  # adjust the spacing
  my $PRI = 0;
  my $W   = 0;
  foreach my $id (@{ $self->{-draworder} }) {
    my $obj = $self->{-id2object}->{$id};
    $W   += $obj->width;
    $PRI += $obj->{-priority} || 0;
  }

  my $width = $self->width;
  my $space = ($width - $W) / (@{ $self->{-draworder} } + 1);
  my $x     = $width;
  foreach my $id (@{ $self->{-draworder} }) {
    my $obj = $self->{-id2object}->{$id};
    $x -= $space + $obj->width;
    $obj->{-x} = int($x);
  }

  return $self->SUPER::layout_contained_objects;
}

package Curses::UI::LayoutVBox;
use base qw(Curses::UI::Container);
$INC{'Curses/UI/LayoutVBox.pm'} = 'internal';

sub add {
  my $self = shift;
  my ($name, $type, %ARGS) = @_;
  my $widget = $self->SUPER::add($name, $type, %ARGS);
  $widget->{-priority} = $ARGS{-priority} if exists $ARGS{-priority};
  $self->layout_contained_objects;
  return $widget;
}

sub layout_contained_objects {
  my $self = shift;

  # adjust the spacing
  my $PRI = 0;
  my $H   = 0;
  foreach my $id (@{ $self->{-draworder} }) {
    my $obj = $self->{-id2object}->{$id};
    $H   += $obj->height;
    $PRI += $obj->{-priority} || 0;
  }

  my $align  = $self->{-align} || 'middle';
  my $width  = $self->width;
  my $height = $self->height;
  my $space  = ($height - $H) / (@{ $self->{-draworder} } + 1);
  my $y      = $height;
  foreach my $id (@{ $self->{-draworder} }) {
    my $obj = $self->{-id2object}->{$id};
    $y -= $space + $obj->height;
    $obj->{-x} = int(($width - $obj->width) / 2) if $align eq 'middle';
    $obj->{-y} = int($y - 1);
  }

  return $self->SUPER::layout_contained_objects;
}

package UI::All::CursesUI::LayoutVBox;
use base qw(UI::All::CursesUI);

sub init_proxy {
  my $self = shift;

  $self->proxy = $self->parent->proxy->add($self->name || $self->gen_name,
                                           'Curses::UI::LayoutVBox',
                                           -border => 1,
                                           @_);
  return $self;
}

package UI::All::CursesUI::LayoutHBox;
use base qw(UI::All::CursesUI);

sub init_proxy {
  my $self = shift;

  $self->proxy = $self->parent->proxy->add($self->name || $self->gen_name,
                                           'Curses::UI::LayoutHBox',
                                           @_);
  return $self;
}

###----------------------------------------------------------------###

package UI::All::CursesUI::Label;
use base qw(UI::All::CursesUI);

sub init_proxy {
  my $self = shift;

  $self->proxy = $self->parent->proxy->add($self->name || $self->gen_name,
                                           'Label',
                                           -width => length($self->{'text'}) + 2,
                                           @_);
  return $self;
}

sub set_text { shift->proxy->text(shift) }

###----------------------------------------------------------------###

package UI::All::CursesUI::LineEdit;
use base qw(UI::All::CursesUI);

use UI::Signals signals => {
  changed        => '1',
  return_pressed => '1',
};

sub init_proxy {
  my $self = shift;

  $self->proxy = $self->parent->proxy->add($self->name || $self->gen_name,
                                           'TextEntry',
                                           -border => 0,
                                           -sbborder => 1,
                                           -bg => 'yellow',
                                           @_);
  return $self;
}

sub set_text { shift->proxy->text(shift) }

###----------------------------------------------------------------###

package UI::All::CursesUI::TextEdit;
use base qw(UI::All::CursesUI);

sub init_proxy {
  my $self = shift;

  $self->proxy = $self->parent->proxy->add($self->name || $self->gen_name,
                                           'TextEditor',
                                           -border => 1,
                                           @_);
  return $self;
}

sub set_text { shift->proxy->text(shift) }

###----------------------------------------------------------------###

package UI::All::CursesUI::Button;
use base qw(UI::All::CursesUI);

sub init_proxy {
  my $self = shift;
  
  $self->proxy = $self->parent->proxy->add($self->name || $self->gen_name,
                                           'Buttonbox',
                                           -border => 0,
                                           -sbborder => 1,
                                           -buttonalignment => 'middle',
                                           -width => length($self->{'text'}) + 4,
                                           -buttons => [{
                                             -label => delete($self->{'text'}),
                                             -onpress => sub { exit },
                                           }],
                                           );
  return $self;
}

sub set_text {
  my $self = shift;
  my $ref = $self->proxy->{-buttons}->[0];
  $ref->{-label} = shift;
  $self->proxy->{-w} = length($ref->{-label}) + 4;
}


###----------------------------------------------------------------###

1;

__END__

=head1 NAME

UI::All::CursesUI - Curses::UI flavor of UI::All

=cut
