package UI::All::Gtk2;

###----------------------------------------------------------------###
#  Copyright 2004 - Paul Seamons                                     #
#  Distributed under the Perl Artistic License without warranty      #
###----------------------------------------------------------------###

use Gtk2;
use base qw(UI::All::Base);

sub attach_proxy {
  my $self = shift;
  my ($slot, $proxy, $signal) = @_;

  my $sig_num = $proxy->signal_connect($signal => sub { $self->$slot(@_) });
  $self->add_property(sig_num => {$slot => {signal => {$proxy => $sig_num}}});
}

###----------------------------------------------------------------###
### layout classes

package UI::All::Gtk2::Layout;
use base qw(UI::All::Gtk2);

sub is_widget { 0 }
sub is_layout { 1 }

sub add_widget {
  my ($layout, $widget, $args) = @_;
  if ($layout->isa('LayoutGrid')) {
    die "No add_widget yet for grid layouts";
  } else {
    $layout->proxy->pack_start($widget->proxy,
                               1, # expand (bool)
                               1, # fill (bool)
                               0, # padding (int)
                               );
  }
}

sub add_spacer {
  my ($layout, $spacer) = @_;
  die; $layout->proxy->addItem($spacer->proxy);
}

#sub set_margin  { confess; shift->proxy->setMargin( shift) }
#sub set_spacing { die; shift->proxy->setSpacing(shift) }

###----------------------------------------------------------------###

package UI::All::Gtk2::LayoutVBox;
use base qw(UI::All::Gtk2::Layout);

sub init_proxy {
  my $self = shift;
  if ($self->parent->is_layout) {
    $self->parent->proxy->add($self->proxy);
  } else {
    my $homogeneous = 0;
    $self->proxy = Gtk2::VBox->new($homogeneous,
                                   defined($self->{'spacing'}) ? $self->{'spacing'} : -1,
                                   );
    $self->parent_widget->proxy->add($self->proxy);
  }
}

###----------------------------------------------------------------###

package UI::All::Gtk2::LayoutHBox;
use base qw(UI::All::Gtk2::Layout);

sub init_proxy {
  my $self = shift;
  $self->proxy = Gtk2::HBox->new($homogeneous,
                                 defined($self->{'spacing'}) ? $self->{'spacing'} : -1,
                                 );
  if ($self->parent->is_layout) {
    $self->parent->proxy->add($self->proxy);
  } else {
    $self->parent_widget->proxy->add($self->proxy);
  }
}



###----------------------------------------------------------------###
### The following are Widget classes supported by UI::All::Gtk2
###----------------------------------------------------------------###

package UI::All::Gtk2::MainWindow;
use base qw(UI::All::Gtk2);

sub init_proxy {
  my $self = shift;
  my $argv = [];
  Gtk2->init;
  $self->proxy = Gtk2::Window->new('toplevel');
}

sub parent { undef } # MainWindows shouldn't have parents

sub set_title { shift->proxy->set_title(shift) }

sub main_loop {
  my $self = shift;

  $self->proxy->show_all;
  Gtk2->main;
}

sub exit_loop {
  my $self = shift;
  Gtk2->main_quit;
  $self->DESTROY;
}

###----------------------------------------------------------------###

package UI::All::Gtk2::Label;
use base qw(UI::All::Gtk2);

sub init_proxy {
  my $self = shift;
  $self->proxy = Gtk2::Label->new('');
  $self->parent_layout->add_widget($self);
}

sub text     { shift->proxy->get_label }
sub set_text { shift->proxy->set_label(shift) }

###----------------------------------------------------------------###

package UI::All::Gtk2::Button;
use base qw(UI::All::Gtk2);
use UI::Signals signals => {
  clicked => 'clicked',
};

sub init_proxy {
  my $self = shift;
  $self->proxy = Gtk2::Button->new('');
  $self->parent_layout->add_widget($self);
}

sub text     { shift->proxy->get_label }
sub set_text { shift->proxy->set_label(shift) }

###----------------------------------------------------------------###

package UI::All::Gtk2::LineEdit;
use base qw(UI::All::Gtk2);
use UI::Signals signals => {
  changed => 'insert_at_cursor',
};

sub init_proxy {
  my $self = shift;
  $self->proxy = Gtk2::Entry->new();
  $self->parent_layout->add_widget($self);
}

sub text     { shift->proxy->get_text }
sub set_text { shift->proxy->set_text(shift) }

###----------------------------------------------------------------###

package UI::All::Gtk2::TextEdit;
use base qw(UI::All::Gtk2);
use UI::Signals signals => {
  changed => 'changed',
  pasted  => 'paste_clipboard',
  return_pressed => 'return_pressed',
};

sub attach_proxy {
  my ($self, $slot, $proxy, $signal) = @_;
  if ($signal eq 'changed') {
    $proxy = $self->buffer;
    $proxy->signal_connect($signal => sub { $self->$slot() });
  } elsif ($signal eq 'pasted') {
    $proxy = $self->view;
    $proxy->signal_connect($signal => sub { $self->$slot() });
  } elsif ($signal eq 'return_pressed') {
    $proxy->signal_connect('TODO' => sub { $self->$slot() });
  }
}

sub init_proxy {
  my $self = shift;
  $self->proxy = Gtk2::Frame->new;

  $self->add_property(scrwin => Gtk2::ScrolledWindow->new);
  $self->add_property(view   => Gtk2::TextView->new);
  $self->add_property(buffer => $self->view->get_buffer);

  $self->proxy->add($self->scrwin);
  $self->scrwin->add($self->view);
  $self->view->set_focus_vadjustment($self->scrwin->get_vadjustment);

  $self->proxy->set_shadow_type('in');
  $self->scrwin->set_policy('automatic', 'automatic'); # automatic scrollbars
  $self->view->set_left_margin(2); # move text away from edge

  $self->parent_layout->add_widget($self);

}

sub text     {
  my $self = shift;
  return $self->buffer->get_text($self->buffer->get_bounds, 0);
}
sub set_text { shift->buffer->set_text(shift) }

###----------------------------------------------------------------###

#while (gtk_events_pending ())
#  gtk_main_iteration ();

1;

__END__

=head1 NAME

UI::All::Gtk2 - Gtk flavor of UI::All

=cut
