package UI::All::Qt3::FileDialog;

use Qt;

{
  package Proxy::FileDialog;
  use Qt::isa qw(Qt::FileDialog UI::All::Qt3);
}

use vars qw(%ACTIONS);
use strict;
use base qw(UI::All::Qt3 UI::All::Base::FileDialog);
Proxy::FileDialog->import;
use UI::Signals signals => {
  file_selected  => 'fileSelected(const QString &)',
  files_selected => 'filesSelected(const QStringList &)',
};

BEGIN {
  %ACTIONS = (
    open_file  => Qt::FileDialog::ExistingFile(),
    open_files => Qt::FileDialog::ExistingFiles(),
    save_file  => Qt::FileDialog::AnyFile(),
    choose_dir => Qt::FileDialog::DirectoryOnly(),
  );
};

sub init_proxy {
  my $self = shift;
  my $parent = $self->parent_widget;

  my $action = delete($self->{'action'}) || '';
  $action    = $self->default_action if ! $self->valid_action($action);
  $self->{'title'} ||= $self->default_title($action);

  $self->proxy = Proxy::FileDialog($parent->proxy,
                                   $self->name || $self->gen_name,
                                   1, # modal
                                   );
  $self->proxy->setMode($ACTIONS{$action});
  $parent->add_child($self);
}

sub set_dir      { shift->proxy->setDir(shift) }
sub set_title    { shift->proxy->setCaption(shift) }

sub get_filename {
  my $self = shift;
  $self->parent_widget->remove_child($self);
  if ($self->proxy->exec == Qt::Dialog::Accepted()) {
    if ($self->proxy->mode == Qt::FileDialog::ExistingFiles()) {
      return $self->proxy->selectedFiles();
    } else {
      return $self->proxy->selectedFile();
    }
  } else {
    return undef;
  }
}

*get_filenames = \&get_filename;

1;
