package UI::All::Qt3::Table;

use Qt;

{
  package Proxy::Table;
  use Qt::isa qw(Qt::Table UI::All::Qt3);
}

use strict;
use base qw(UI::All::Qt3);
Proxy::Table->import;
use UI::Signals signals => {
};

sub init_proxy {
  my $self = shift;

  $self->proxy = Proxy::Table($self->parent_widget->proxy,
                              $self->name || $self->gen_name,
                              );
  $self->parent_layout->add_widget($self);
  $self->add_property('items', {});
}

sub set_rows { shift->proxy->setNumRows(shift) }
sub set_cols { shift->proxy->setNumCols(shift) }

sub set_item {
  my ($self, $row, $col, $item) = @_;
  die "Parent does not equal item's parent" if $self ne $item->parent_widget;
  do { require Carp; Carp::confess("Missing row ($row, $col, $item)") } if ! defined $row;
  $self->items->{$row}->{$col} = $item; # trading speed for memory of sparse
  $self->proxy->setItem($row, $col, $item->proxy);
}
sub item {
  my ($self, $row, $col) = @_;
  return $self->items->{$row}->{$col} ||= $self->TableItem(row => $row, col => $col);
}


sub set_text {
  my ($self, $row, $col, $text) = @_;
  $self->item($row, $col)->set_text($text);
}
sub text { shift->item(shift, shift)->text }


sub set_pixmap {
  my ($self, $row, $col, $pixmap) = @_;
  $self->item($row, $col)->set_pixmap($pixmap);
}
sub pixmap { shift->item(shift, shift)->pixmap }


sub put {
  my ($self, $row, $col, $what) = @_;
  if (! ref $what) {
    $self->set_text($row, $col, $what);
  } elsif ($what->can('is_pixmap') && $what->is_pixmap) {
    $self->set_pixmap($row, $col, $what);
  } elsif ($what->can('is_tableitem') && $what->is_tableitem) {
    $self->set_item($row, $col, $what);
  } else {
    $self->set_text($row, $col, "$what");
  }
}
sub get { shift->item(shift, shift) }

###----------------------------------------------------------------###

package UI::All::Qt3::TableItem;
use base qw(UI::All::Qt3);

sub init_proxy {
  my $self   = shift;
  $self->proxy = Qt::TableItem($self->parent->proxy,
                               Qt::TableItem::OnTyping()
                               );
  $self->parent->set_item(delete $self->{'row'},
                          delete $self->{'col'},
                          $self);
}

sub text   { shift->proxy->text }
sub pixmap { shift->proxy->pixmap }
sub set_text   {
  my ($self, $text) = @_;
  $self->proxy->setText($text);
  $self->parent->proxy->updateCell($self->proxy->row, $self->proxy->col);
}
sub set_pixmap { shift->proxy->setPixmap(shift->proxy) }

###----------------------------------------------------------------###

1;
