package UI::All::Tk::FileDialog;

use strict;
use base qw(UI::All::Tk UI::All::Base::FileDialog);
use UI::Signals signals => {
  file_selected  => 'fileSelected(const QString &)',
  files_selected => 'filesSelected(const QStringList &)',
};

sub attach_proxy {}

sub init_proxy {
  my $self = shift;
  my $action = delete($self->{'action'}) || '';
  $action    = $self->default_action if ! $self->valid_action($action);
  $self->{'title'} ||= $self->default_title($action);

  $self->parent->add_child($self);

  $self->add_property('dir');
  $self->add_property('file');
  $self->add_property('title');
  $self->add_property('file_types');
  $self->add_property('action', $action);
}

sub set_dir        { shift->dir(shift)        }
sub set_file       { shift->file(shift)       }
sub set_title      { shift->title(shift)      }
sub set_file_types { shift->file_types(shift) }

sub get_filename {
  my $self = shift;
  my $proxy = $self->parent->proxy;
  my $return;
  if ($self->action eq 'save_file') {
    $return = $proxy->getSaveFile('-initialdir'  => $self->dir,
                                  '-initialfile' => $self->file,
                                  '-title'       => $self->title,
                                  '-filetypes'   => $self->file_types || [['All Files', '*']],
                                  );
  } elsif ($self->action eq 'choose_dir') {
    die "No choose_dir in Tk yet";
  } else {
    $return = $proxy->getOpenFile('-initialdir'  => $self->dir,
                                  '-initialfile' => $self->file,
                                  '-title'       => $self->title,
                                  '-multiple'    => $self->action eq 'open_files',
                                  '-filetypes'   => $self->file_types || [['All Files', '*']],
                                  );
    if ($return) {
      if ($self->action eq 'open_files') {
        $self->files_selected($return);
      } else {
        $self->file_selected($return);
      }
    }
  }

  $self->parent->remove_child($self);
  return $return;
}

*get_filenames = \&get_filename;

1;
