#!/usr/bin/perl -w

use UI::All;

# Original version of ptadmin.txt written in Tk by steve@thrish.com
# Copyright (C) 2001 Stephen W. Anderson
# GNU General Public License for more details.

my $computer = `hostname`;
chomp($computer);


my $mw   = UI::All->MainWindow(title => $computer .' - '. localtime);
my $vbox = $mw->LayoutVBox;
#$mw->geometry("680x375+0+0");
#$mw->iconname("ptadmin");
#$mw->iconmask();
#$mw->iconmask("info");
#$mw->client();
#$mw->client("ptadmin");

#my $f = $mw->Frame(-highlightbackground => 'blue',
#                   -highlightthickness => 4)->pack(-side => 'top');

###----------------------------------------------------------------###

my $lb = $vbox->ListBox(width => 100, height => 5);
refresh_lb();

###----------------------------------------------------------------###

my $hbox = $vbox->LayoutHBox;

my $lb2 = $hbox->ListBox(height => 5, width => 60);
refresh_lb2();

my $lb3 = $hbox->ListBox(height => 5, width => 40);
refresh_lb3();

###----------------------------------------------------------------###

my $lb4 = $vbox->ListBox(width => 100, height => 5);
refresh_lb4();

###----------------------------------------------------------------###

#   my $g = $mw->Frame(-highlightbackground => 'blue',
#      -highlightthickness => 4);

my $bbox = $vbox->LayoutHBox;

$bbox->Button(text => "Exit",
              clicked => sub { $mw->exit_loop },
              );

$bbox->Button(text => "Ping",
              clicked => \&ping,
              );

$bbox->Button(text => "Kill Process",
              clicked => \&kill_proc,
              );

$bbox->Button(text => "Refresh",
              clicked => sub {
                refresh_lb();
                refresh_lb2();
                refresh_lb3();
                refresh_lb4();
              },
              );

$bbox->Button(text => "Hosts",
              clicked => \&refresh_lb2,
              );

$bbox->Button(text => "Add Host",
              clicked => \&add_host,
              );

$bbox->Button(text => "Delete Host",
              clicked => \&del_host,
              );

$bbox->Button(text => "Add User",
              clicked => \&add_user,
              );

$bbox->Button(text => "Delete User",
              clicked => \&del_user,
              );

$mw->main_loop;

###----------------------------------------------------------------###


sub refresh_lb {
  my @dfk = `df -k`;
  chomp foreach @dfk;
  $lb->delete(0, $lb->last);
  $lb->insert(\@dfk);

  foreach my $part_disk (@dfk){
    my @part_use = split (/\s+/, $part_disk);
    if ($part_use[4] =~ /([\d.]+)%/ && $1 > 95) {
      $lb->insert("Warning! $part_use[0] Usage Percentage exceeds 95");
    }
  }
}

sub refresh_lb2 {
  my @hosts = `cat /etc/hosts`;
  chomp foreach @hosts;
  s/\t/ /g foreach @hosts;

  $lb2->delete(0, $lb2->last);
  $lb2->insert(\@hosts);
}

sub refresh_lb3 {
  my @users = `cat /etc/passwd | awk -F : '{if (\$3 > 99) print \$1"  "  \$3"  " \$4"  " \$5}'`;
  chomp foreach @users;
  $lb3->delete(0, $lb3->last);
  $lb3->insert(\@users);
}

sub refresh_lb4 {
  my @proc = `ps -ef`;
  chomp foreach @proc;
  $lb4->delete(0, $lb4->last);
  $lb4->insert(\@proc);
}

###----------------------------------------------------------------###

sub ping {
  if (! $lb2->has_selection) {
    print "Select a host to Ping!\n";
    return;
  }

  my $selected1 = $lb2->get($lb2->selection);
  my @ip = split (/ /, $selected1);
  chomp $ip[0];
  $lb2->delete(0, $lb2->last);
  if (! `ping -c 1 $ip[0]`) { 
    $lb2->insert("IP appears to be invalid or network is unreachable.");
    return;
  }
  my @ping_results = `ping -c 3 $ip[0]`;
  chomp foreach @ping_results;
  $lb2->insert(\@ping_results);
}

sub kill_proc {
  if (! $lb4->has_selection) {
    print "Select a process to kill!\n";
    return;
  }

  my $selected4 = $lb4->get($lb4->selection);
  my @line = split (/\s+/,$selected4);
  kill(9, $line[1]) || print "Couldn't kill $line[1]: $!\n";

  refresh_lb4();
}

###----------------------------------------------------------------###

sub add_host {
  my $d = $mw->Dialog(title => "ptadmin - Add Host");
  #$d->geometry("290x210+0+0");
  #$b1 = $d->Frame(-highlightbackground => 'blue',
  #                -highlightthickness => 4)->pack(side=>'top', -fill => 'x');
  #$b2 = $b1->Frame(-highlightbackground => 'blue',
  #                 -highlightthickness => 4)->pack(side=>'top', -fill => 'x');

  my $vbox = $d->LayoutVBox;

  $vbox->Label(text => "Host Name");
  my $le_host = $vbox->LineEdit(width => 25);
                                #textvariable => \$hostname) 
  $vbox->Label(text => "IP Address");
  my $le_addr = $vbox->LineEdit(width => 16);

  $vbox->Label(text => "Fully Qualified Name (hostname.domainname)");
  my $le_fqdn = $vbox->LineEdit(width => 25);

  $vbox->Label(text => "Alias");
  my $le_alias = $vbox->LineEdit(width => 15);

  my $hbox = $vbox->LayoutHBox;

  $hbox->Button(text => "Cancel",
                clicked => sub { $d->close },
                );
  $hbox->Button(text => "Add the Host",
                clicked => sub { my @info = ($le_host->text,
                                             $le_addr->text,
                                             $le_fqdn->text,
                                             $le_alias->text
                                             );
                                 $d->close;
                                 add_host_end(@info);
                               },
                );
  $d->start;
}

sub add_host_end {
  my ($hostname, $ipaddress, $fqdn, $alias) = @_;
  $line_cnt=`cat /etc/hosts|wc -l`;
  open(OLD, "</etc/hosts") ||print "Could not open hosts";
  open (NEW, ">/etc/hosts.txt")||print "Could not open hosts.txt";
  select (NEW);
  while (<OLD>) {
    print NEW $_;
    if ($. == $line_cnt && $ipaddress ne "") {
      print NEW "$ipaddress\t$hostname\t$fqdn\t$alias\n"; 
    }
  }
  close (OLD);
  close (NEW);
  select (STDOUT);
  rename ("/etc/hosts", "/etc/hosts.orig")||print "HELP1";
  rename("/etc/hosts.txt", "/etc/hosts")||print "HELP";

  refresh_lb2();
}

sub del_host {
  if (! $lb2->has_selection) {
    print "Select Host!";
    return;
  }

  my $selected1 = $lb2->get($lb2->selection);
  my $line_cnt  = `cat /etc/hosts|wc -l`;
  my ($new, $old);
  if (! open ($old, "</etc/hosts")
      || ! open ($new, ">/etc/hosts.temp")) {
    print "Couldn't open /etc/hosts or hosts.temp: $!\n";
    return;
  }

  while (<$old>) {
    chomp;
    next if $_ eq $selected1;
    print $new $_ ."\n";
  }

  rename("/etc/hosts", "/etc/hosts.orig") || print "Couldn't rename /etc/hosts to /etc/hosts.orig: $!\n";
  rename("/etc/hosts.temp", "/etc/hosts") || print "Couldn't rename /etc/hosts.temp to /etc/hosts: $!\n";

  refresh_lb2();
}

###----------------------------------------------------------------###

sub add_user {
  if (! Exists ($UserAdd)) {
    $UserAdd = $mw->Toplevel();
    $UserAdd ->title("ptadmin - Add User");
    $UserAdd->geometry("240x215+0+0");
    $b1 = $UserAdd->Frame(-highlightbackground => 'blue',
                          -highlightthickness => 4)->pack(side=>'top', -fill => 'x');
    $b2 = $b1->Frame(-highlightbackground => 'blue',
                     -highlightthickness => 4)->pack(side=>'top', -fill => 'x');
    $b2->Label(-text => 'Login (Required)' ) ->pack(-side => 'top');
    $b2->Entry(-width => 25, -textvariable => \$login) 
      ->pack(-side => 'top');
    $b2->Label(-text => 'User Name (Optional)' ) ->pack(-side => 'top');
    $b2->Entry(-width => 25, -textvariable => \$username) 
      ->pack(-side => 'top');
    $b2->Label(-text => 'Password (Required)' ) ->pack(-side => 'top');
    $b2->Entry(-width => 25, -show => "*", 
               -textvariable => \$password) ->pack(-side => 'top');
    $b2->Label(-text => 'Home Directory (Required)' ) ->pack(-side => 'top');
    $b2->Entry(-width => 25, -textvariable => \$homedir) 
      ->pack(-side => 'top');
    $b1->Button(-text => "Add the User", -command => \&add_user_end)
      ->pack(-side => 'right');
    $b1->Button(-text => "Cancel", -command => sub {$UserAdd->destroy})
      ->pack(-side => 'right');
  } else {
    $UserAdd->raise();
  }
}

sub add_user_end {
  `/usr/sbin/useradd -p $password -c "$username" -d $homedir $login`;

  refresh_lb3();
}

sub del_user {
  if (!$lb3->selectionIncludes('active')){
    print "Select User!";
  }
  else{
    if (! Exists ($UserWarn)) {
      $UserWarn = $mw->Toplevel();
      $UserWarn ->title("ptadmin Warning!");
      $UserWarn->geometry("200x80+0+0");
      $b1 = $UserWarn->Frame(-highlightbackground => 'blue',
                             -highlightthickness => 4)->pack(side=>'top', -fill => 'x');
      $b2 = $b1->Frame(-highlightbackground => 'blue',
                       -highlightthickness => 4)->pack(side=>'top', -fill => 'x');
      $b2->Label(-text => "Warning! User directories \n will be permanently removed!" ) ->pack(-side => 'top');
      $b1->Button(-text => "Remove the User!", -command => \&del_user_end)
        ->pack(-side => 'right');
      $b1->Button(-text => "Cancel", -command => sub {$UserWarn->destroy})
        ->pack(-side => 'right');
    } else {
      $UserWarn->raise();
    }
  }
}

sub del_user_end {
  my $selected3 = $lb3->get($lb3->curselection());
  my @login = split (/\s+/, $selected3);
  `/usr/sbin/userdel -r $login[0]`;

  refresh_lb3();
}
1;
