#!/usr/bin/perl -w

  use strict;

  ###-----------------------------------------------###

  ### setup some nice classes
  {
    package Foo;
    use UI::Signals qw(attach detach);
    use UI::Signals signals => {add_it => 'number'};
    sub new {
      my $class = shift || die;
      my $n     = shift || 0;
      bless \ $n, $class;
    }
    sub n :lvalue { ${ +shift } }
    sub add { shift->n += shift || 0 }
  }
  {
    package Bar;
    @Bar::ISA = qw(Foo);
  }
  {
    package Baz;
    @Baz::ISA = qw(Bar);
  }

  ###-----------------------------------------------###
  ### create some objects

  my $foo = Foo->new(1);
  my $bar = Bar->new(2);
  my $baz = Baz->new(3);
  printf '$foo->n = %d, $bar->n = %d, $baz->n = %d'."\n",
    $foo->n, $bar->n, $baz->n;
  # prints '$foo->n = 1, $bar->n = 2, $baz->n = 3';


  ###-----------------------------------------------###
  ### finally - setup some connections

  ### attach by method name
  $foo->attach('add_it', $foo, 'add');
    # OR UI::Signals::attach($foo, 'add_it', $foo, 'add');

  ### attach by method coderef
  $foo->attach('add_it', $bar, sub {my $self = shift; $self->n += shift});

  ### attach an anonymous sub
  $foo->attach('add_it', undef, sub {$baz->n += shift}); # code ref

  ### try out our signals
  $foo->add_it(3);

  printf '$foo->n = %d, $bar->n = %d, $baz->n = %d'."\n",
    $foo->n, $bar->n, $baz->n;
  # prints '$foo->n = 4, $bar->n = 5, $baz->n = 6';
  # UI::Signals->dump_signals;


  ###-----------------------------------------------###
  ### now we can detach connections

  ### detach with specific info
  $foo->detach('add_it', $foo);
    # OR UI::Signals::detach($foo, 'add_it, $foo);

  ### detach all connections of bar to 'add_it'
  UI::Signals::detach(undef, 'add_it', $bar);

  ### detach all connections of bar to anything
  UI::Signals::detach(undef, undef, $bar);

  ### detach last anonymous sub from foo add_it
  $foo->detach('add_it', '');    # defined but false

  ### detach all connections from foo add_it
  $foo->detach('add_it', undef); # undefined and false

  ### detach all connections from foo
  $foo->detach;

  ### see if it worked
  $foo->add_it(3);

  printf '$foo->n = %d, $bar->n = %d, $baz->n = %d'."\n",
    $foo->n, $bar->n, $baz->n;
  # still prints '$foo->n = 4, $bar->n = 5, $baz->n = 6';
  # UI::Signals->dump_signals;

  ###-----------------------------------------------###

  exit;
