#!/usr/bin/perl -w

package Walker;

sub new {
  my ($class, @images) = @_;
  my $dir = __FILE__;
  $dir =~ s|/[^/]+$||;
  my @frames = map { SDL::Surface->new(-name => "$dir/tutorial4_$name.png") };
  my $frame_rect = SDL::Rect->new(
                                  -height => $frames[0]->height(),
                                  -width  => $frames[0]->width(),
                                  -x      => 0,
                                  -y      => 0,
                                  );
  my $self = {
    frames     => \@frames,
    frame_rect => $frame_rect,
  };
  bless $self, $class;
}

sub frames
{
  my $self        = shift;
  $self->{frames} = shift if @_;
  $self->{frames};
}

sub frame_rect
{
  my $self            = shift;
  $self->{frame_rect} = shift if @_;
  $self->{frame_rect};
}

sub next_frame
{
  my $self   = shift;
  my $frames = $self->frames();
  my $frame  = shift @$frames;

  push @$frames, $frame;
  $self->frames( $frames );

  return $frame;
}

###----------------------------------------------------------------###

package main;

use strict;

use SDL;
use SDL::App;
use SDL::Surface;
use SDL::Color;

# change these values as necessary
my $title                                = 'My SDL Animation';
my ($width,      $height,      $depth)   = (  640,  480,   16 );
my ($bg_r,       $bg_g,        $bg_b)    = ( 0xff, 0xff, 0xff );
my ($start_x,    $end_x)                 = (   20,  600       );
my $sleep_msec                           = 0.05;

my $app = SDL::App->new(
                        -width  => $width,
                        -height => $height,
                        -depth  => $depth,
                        );

my $bg_color = SDL::Color->new(
                               -r => $bg_r,
                               -g => $bg_g,
                               -b => $bg_b,
                               );

my $background = SDL::Rect->new(
                                -width  => $width,
                                -height => $height,
                                );

my $pos = SDL::Rect->new(
                         -width  => 16,
                         -height => 16,
                         -x      => 0,
                         -y      => 240,
                         );

my $walker = Walker->new(qw(left center right));

for my $x ( $start_x .. $end_x ) {
  draw_background( $app, $background, $bg_color );
  $pos->x( $x );
  draw_walker( $walker, $app, $pos );
  $app->update( $background );
  select( undef, undef, undef, $sleep_msec );
}

# you'll want to remove this
sleep 2;

sub draw_background
{
  my ($app, $background, $bg_color) = @_;
  $app->fill( $background, $bg_color );
}

sub draw_walker
{
  my ($walker, $app, $pos) = @_;
  my $frame                = $walker->next_frame();
  my $frame_rect           = $walker->frame_rect();
  $frame->blit( $frame_rect, $app, $pos );
}


