#!/usr/bin/perl -w

{
  package Foo;
  sub new {
    my $class = shift || __PACKAGE__;
    my $n     = shift || 0;
    bless \ $n, $class;
  }
  sub add { ${ +shift } += shift || 0 }
  sub n :lvalue { ${ +shift } }
}
{
  package Bar;
  @Bar::ISA = qw(Foo);
}
{
  package Baz;
  @Baz::ISA = qw(Bar);
}

###----------------------------------------------------------------###

use strict;
use Test::More tests => 10;

use_ok('UI::Signals');

ok(UI::Signals->import qw(attach detach), "Import 1");
ok(! eval {UI::Signals->import qw(foo)}, "Import 3");

{
  package Foo;
  UI::Signals->import(signals => {
    add_it => ['number'],
  });
}

my $foo = Foo->new(1);
my $bar = Bar->new(2);
my $baz = Baz->new(3);

ok($bar->n == 2, "Foo worked");
ok($baz->add(3) == 6, "Add works");
$foo->n = $bar->n = $baz->n = 0;
ok($baz->n == 0, "Cleared");

### attach three subs to signals
attach($foo, 'add_it', $foo, 'add'); # method name
attach($foo, 'add_it', $bar, sub {my $self = shift; $self->n = shift}); # code method
attach($foo, 'add_it', undef, sub {$baz->n = shift}); # code ref

$foo->add_it(2);
ok($foo->n == 2 && $bar->n == 2 && $baz->n == 2, "Signal Test");
$foo->n = $bar->n = $baz->n = 0;

#UI::Signals->dump_signals;
detach($foo, 'add_it', $foo);
$foo->add_it(2);
ok($foo->n == 0 && $bar->n == 2 && $baz->n == 2, "Signal Test 2");
$foo->n = $bar->n = $baz->n = 0;

detach(undef, 'add_it', $bar);
$foo->add_it(2);
ok($foo->n == 0 && $bar->n == 0 && $baz->n == 2, "Signal Test 3");
$foo->n = $bar->n = $baz->n = 0;

detach($foo, 'add_it');
$foo->add_it(2);
ok($foo->n == 0 && $bar->n == 0 && $baz->n == 0, "Signal Test 4");
$foo->n = $bar->n = $baz->n = 0;

#UI::Signals->dump_signals;
exit;

